;=========================================================================
;Bernd Kernbaum
;DK3WX
;=========================================================================
;
;PIC 4 MHz Quarztakt
;Funktion:
;
;=========================================================================
;Konstanten fuer die PIC-Konfiguration
;=========================================================================
#define ptype	873			; 873 oder 876

;********************************
	if ptype == 876
;********************************

	list      p=16f876             ; list directive to define processor
	#include <p16f876.inc>         ; processor specific variable definitions
	__config   _cp_off & _lvp_off & _wdt_off & _pwrte_on & _hs_osc

;********************************
	else
;********************************

	list      p=16f873             ; list directive to define processor
	#include <p16f873.inc>         ; processor specific variable definitions
	__config   _cp_off & _lve_off & _wdt_off & _pwrte_on & _hs_osc

;********************************
	endif
;********************************


;==========================================================
; Variablen:
;==========================================================
        org     020h

rette_w:	res     1	; W retten in der ISR  
rette_f:	res     1     	; f retten in der ISR
rette_p:	res     1     	; pclath retten in der ISR


bcounter:	res	1	;Bitzaehler 	
sr:		res	1	;Empfangsschieberegister
temp:		res	1	;Zaehler fuer Warteschleifen
temp1:		res	1	;Zaehler fuer Warteschleifen
werth:		res	1	;Messwert des AD Wandlers
wertl:		res	1	;Messwert des AD Wandlers
	
f_b0:		res	1      	;Frequenzspeicher zur dualen Rechnung
f_b1:		res	1      	;"
f_b2:		res	1      	;"
f_b3:		res	1      	;"
f_b4:		res	1      	;Konfigurationsbyte DDS-IC 

sr0:		res	1      	;Schieberegister
sr1:		res	1      	;"
sr2:		res	1      	;"
sr3:		res	1      	;"
sr4:		res	1      	;"

;-----------------------------------------------------------------------------
;Variablen Bootloader
		org	20h
addressh:	res	1	;flash program memory address high byte
addressl:	res	1	;flash program memory address low byte
numwords:	res	1	;number of words in line of hex file
checksum:	res	1	;byte to hold checksum of incoming data
counter:	res	1	;to count words being saved or programmed
testbyte:	res	1	;byte to show reset vector code received
hexbyte:	res	1	;byte from 2 incoming ascii characters
datapointer:	res	1	;pointer to data in buffer
dataarray:	res	0x40	;buffer for storing incoming data


;=========================================================================
;Namen der I/O Leitungen 
;=========================================================================

;port a
;#define e_mess_a0	porta,0h        ;Analogeingang log.
;#define e_mess_a1	porta,1h        ;Analogeingang lin.
;#define e_mess_a2	porta,2h        ;Analogeingang 
;#define e_mess_a3	porta,3h        ;Analogeingang 


;port b mode Taste
#define e_typesel   	portb,0h        ;DDS Type whlen
#define e_pictakt   	portb,1h        ;Taktfrequenz PIC 
#define e_baud   	portb,2h        ;Baudrate whlen
#define e_version0   	portb,3h        ;Software Version 
#define e_version1   	portb,4h        ;Software Version 



;port c

#define a_data  	portc,0h        ;Daten DDS
#define a_reset   	portc,1h        ;reset DDS
#define a_fqud	  	portc,2h        ;Latch DDS
#define a_takt	  	portc,3h        ;Takt DDS
#define a_rts   	portc,4h        ;RTS_OUTPUT
#define e_cts   	portc,5h	;CTS_INPUT
#define a_sio		portc,6h   	;Daten zum PC
#define e_sio   	portc,7h	;Datenempfang vom PC


;-----------------------------------------------------------------------------
;Konstanten Bootloader

#define test_input	portb,3h	;port b pin input indicates download
#define rts_output	portc,4h	;port b pin 1 output for flow control
#define cts_input	portc,5h	;port b pin 2 input for flow control


;=========================================================================
;Konstanten fuer die PIC-Konfiguration
;=========================================================================
d_option_reg:       	equ   	00h
d_port_a:           	equ   	00h
d_port_b:           	equ   	00h
d_port_c:           	equ   	00h
d_tmr0:             	equ   	00h
						;0=Ausgang
						;1=Eingang
d_trisa:            	equ   	b'11111111'	;Funktion siehe Definitionen

d_trisb:            	equ   	b'11111111' 

d_trisc:            	equ   	b'10110000'

d_intcon:           	equ   	00h		;global interrupt disable
d_adcon1:           	equ   	00h    		;PortA als 5 analog. Eingaenge  
						; ..linksbuendig
d_t1con:		equ	b'00100001'	;Timer 1 intern VT8 go
d_pie1:			equ	b'00000000'	;Timer 1 enable
d_pie2:			equ	b'00000000'	


						;Fosz ist  4MHz
baud_9600		equ	0x19		;Konstante fuer 9600 Baud
baud_19200		equ	0x0c		;Konstante fuer 19200 Baud
						;Fosz ist  10MHz
baud_38400		equ	0x0f		;Konstante fuer 38400 Baud
baud_57600		equ	0x0a		;Konstante fuer 57600 Baud

bank0:              	equ   	00h
bank1:              	equ   	80h
bank2:              	equ   	100h
bank3:              	equ   	180h


;=========================================================================
; Hardware initialisieren
;=========================================================================
;reset vector code

	org	0x0000 

resetvector:
	movlw	high main
	movwf	pclath		;set page bits for page3
	goto    main		;go to boot loader

anf:    
        clrf        status        ; Flags loeschen
            
        banksel bank1             ; Registerbank 80h..0AFh
        movlw       d_option_reg
        movwf       option_reg
	movlw	    d_pie1
        movwf       pie1
	movlw	    d_pie2
        movwf       pie2
        movlw       d_trisa
        movwf       trisa
        movlw       d_trisb
        movwf       trisb
        movlw       d_trisc
        movwf       trisc
        movlw       d_adcon1
        movwf       adcon1
        banksel bank0             ; Registerbank 0..2FH
          

        movlw       d_port_a
        movwf       porta
        movlw       d_port_b
        movwf       portb
        movlw       d_port_c
        movwf       portc
        movlw       d_intcon
        movwf       intcon
	movlw	    d_t1con
	movwf	    t1con
	clrf	pir1
	clrf	ccp1con
	clrf	ccp2con

;=========================================================================
;Hauptprogramm
;=========================================================================

haupt:	
	call	serialsetup
	movlw	28h		; 1 MHz Sinus bei 100 MHz Takt
	movwf	f_b0
	movlw	5ch
	movwf	f_b1
	movlw	8fh
	movwf	f_b2
	movlw	02h
	movwf	f_b3
	clrf	f_b4
	btfsc	e_typesel	; Jumper Type select abfragen  
	bsf	f_b4,0

	bsf	a_reset		; RESET 
	nop
	nop
	nop
	nop
	nop
	nop
	bcf	a_reset
	nop
	nop
	nop
	nop
	nop
	bsf	a_takt		; Umschalten auf serieller Eingabe
	nop			; D0,D1 high, D2 bis D6 low, D7 Dateneingang
	nop
	bcf	a_takt
	nop
	nop
	bsf	a_fqud
	nop
	nop
	bcf	a_fqud
	nop
	nop
	
	call	ausg        	;erste ausgabe an AD9850/1
	call	ausg	


hloop:	call	rxd		;Byte 0
	movwf	f_b0
	call	rxd		;Byte 1
	movwf	f_b1
	call	rxd		;Byte 2
	movwf	f_b2
	call	rxd		;Byte 3
	movwf	f_b3

	call	ausg		; Frequenz an den DDS Chip

	call	rxd		; warten auf den Messbeginn
	
	movwf	temp
	andlw	0f0h
	xorlw	50h
	btfsc	status,z
	goto	hloop0
	call	ad0mess8	; Messen 
	call	txd		; Messwert zum PC
	goto	hloop

hloop0:
	movf	temp,w
	andlw	07h
	addwf	pcl,f
	goto	hloop1		;50h AD0 8Bit
	goto	hloop2		;51h AD1 8Bit
	goto	hloop3		;52h AD2 8Bit
	goto	hloop4		;53h AD3 8Bit
	goto	hloop5		;54h AD0 10Bit
	goto	hloop6		;55h AD1 10Bit
	goto	hloop7		;56h AD2 10Bit
	goto	hloop8		;57h AD3 10Bit
	goto	hloop

hloop1:
	movlw	041h
	call	adxmess
	call	txd
	goto	hloop
hloop2:
	movlw	049h
	call	adxmess
	call	txd
	goto	hloop
hloop3:
	movlw	051h
	call	adxmess
	call	txd
	goto	hloop
hloop4:
	movlw	059h
	call	adxmess
	call	txd
	goto	hloop
hloop5:
	movlw	041h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop6:
	movlw	049h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop7:
	movlw	051h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop
hloop8:
	movlw	059h
	call	adxmess
	call	txd
	movf	wertl,w
	call	txd
	goto	hloop


;=========================================================================
;Initialisieren der UART Port RC4 bis RC7 
;=========================================================================

serialsetup:
	banksel	bank0		;
	bsf	a_rts           ;RTS aus
	banksel	bank1		;
	bcf	trisc,4         ;RTS Pin wird Ausgang

	btfss	e_pictakt	;mit welcher Taktfrequenz arbeitet der PIC?
	goto	takt4mhz

takt10mhz:
				;10 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_38400	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_57600	;Setze Baudrate
	goto	setbaudrate

takt4mhz:
				;4 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_9600	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_19200	;Setze Baudrate

setbaudrate:
	movwf	spbrg
	bsf	txsta,brgh	;baud rate high speed option
	bsf	txsta,txen	;enable transmission
	banksel	bank0		;
	bsf	rcsta,cren	;enable reception
	bsf	rcsta,spen	;enable serial port
	return


;=========================================================================
;serieller Empfang von 8 Bit an RC7 
;=========================================================================
rxd:
	bcf	a_rts           ;setze RTS fuer Datenempfang
	btfss	pir1,rcif	;test ob Daten 
	goto	$-1		;..wenn nein, dann warten
	movf	rcreg,w		;Daten ins W-Register
	return

;=========================================================================
;Senden von 8 Bit an RC6 
;=========================================================================
txd_cts:
	btfsc	e_cts	        ;teste CTS ob gesendet werden kann
	goto	$-1
txd:
	btfss	pir1,txif	;.. und ob der Sendepuffer leer ist
	goto	$-1
	movwf	txreg		;Sende Byte
	return


;=========================================================================
;A/D Wandler Wert abfragen
;=========================================================================
ad0mess8:
	movlw	0x41     	; anal. kanal RA0
adxmess:
	movwf	adcon0		; ... als Messeingang setzen
	call	zem		;kurz warten
	bsf	adcon0,2	;Start Messung
ad0me1:	btfsc	adcon0,2	;warten bis Messung beendet
	goto	ad0me1   	;Ergebnis in adresh und adresl
	banksel	bank1
	movf	adresl,w
	banksel	bank0
	movwf	wertl
	movf	adresh,w	;nur die oberen 8 Bit werden genutzt
	movwf	werth
	return

zem:	movlw	0x07   		; zeitschleife 26 us  
	movwf	temp
zem1:	decfsz	temp,f
	goto	zem1
	return


;=========================================================================
;Umladen ins Schieberegister
;=========================================================================
ausg:   bcf         a_fqud      ;umladen
        bcf         a_takt
        bcf         a_data
        movf        f_b0,w
        movwf       sr0
        movf        f_b1,w
        movwf       sr1
        movf        f_b2,w
        movwf       sr2
        movf        f_b3,w
        movwf       sr3
        movf        f_b4,w
        movwf       sr4

;=========================================================================
;Ausgabe an Frequenzregister des AD9850
;=========================================================================
ausg0:  movlw       28h		;40 dez Bit
        movwf       bcounter
rot:    bcf         a_takt
        bcf         status,0
        rrf         sr4,f
        rrf         sr3,f
        rrf         sr2,f
        rrf         sr1,f
        rrf         sr0,f
        btfss       status,0
        goto        rot0
        bsf         a_data
        goto        rotw
rot0:   bcf         a_data
rotw:   nop
        bsf         a_takt
        decfsz      bcounter,f
        goto        rot
        bcf         a_takt
        nop
        bsf         a_fqud
        nop
        nop
        nop
        nop
        bcf         a_fqud
        nop
        nop
        nop
        nop
        bcf         a_takt
        bcf         a_data
        return



;=============================================================================
; software license agreement
;
; the software supplied herewith by microchip technology incorporated 
; (the "company") for its picmicro microcontroller is intended and 
; supplied to you, the companys customer, for use solely and 
; exclusively on microchip picmicro microcontroller products. the 
; software is owned by the company and/or its supplier, and is 
; protected under applicable copyright laws. all rights are reserved. 
; any use in violation of the foregoing restrictions may subject the 
; user to criminal sanctions under applicable laws, as well as to 
; civil liability for the breach of the terms and conditions of this 
; license.
;
; this software is provided in an "as is" condition. no warranties, 
; whether express, implied or statutory, including, but not limited 
; to, implied warranties of merchantability and fitness for a 
; particular purpose apply to this software. the company shall not, 
; in any circumstances, be liable for special, incidental or 
; consequential damages, for any reason whatsoever.
;
;=============================================================================
;	filename:	boot877.asm
;=============================================================================
;	author:		mike garbutt
;	company:	microchip technology inc.
;	revision:	1.00
;	date:		26 june 2000
;	assembled using mpasm v2.40
;=============================================================================
;	include files:	p16f877.inc	v1.00
;=============================================================================
;	boot code to receive a hex file containing user code from a
;	serial port and write it to program memory. tests a pin to see
;	if code should be downloaded. receives hex file using usart and
;	hardware handshaking. does error checking on data and writes to
;	program memory. waits for reset and then starts user code running.
;=============================================================================
;start of boot code in upper memory traps accidental entry into boot code area

;		org	0x1f00		;use last part of page3 for pic16f876/7
		org	0x0f00		;use last part of page1 for pic16f873/4
;		org	0x0700		;use last part of page0 for pic16f870/1

startofboot:	movlw	high traperror	;trap if execution runs into boot code
		movwf	pclath		;set correct page
traperror:	goto	traperror	;trap error and wait for reset

;-----------------------------------------------------------------------------
;relocated user reset code to jump to start of user code
;must be in bank0 before jumping to this routine

startusercode:
		clrf	pclath		;set correct page for reset condition 
		goto	anf		;relocated user code replaces this nop
		nop			;relocated user code replaces this nop
		nop			;relocated user code replaces this nop
		nop			;relocated user code replaces this nop
		movlw	high traperror1	;trap if no goto in user reset code
		movwf	pclath		;set correct page
traperror1:	goto	traperror1	;trap error and wait for reset

;-----------------------------------------------------------------------------
;program memory location to show whether valid code has been programmed

codestatus:	da	0x0000		;0 for valid code, 0x3fff for no code

;-----------------------------------------------------------------------------
;main boot code routine
;tests to see if a load should occur and if valid user code exists

main:
		banksel	bank1
		clrf	option_reg
		banksel	bank0		;change to bank0 in case of soft reset
		btfss	test_input 	;check pin for boot load		
		goto	loader		;if low then do bootload
		call	loadstatusaddr	;load address of codestatus word
		call	flashread	;read data at codestatus location
		banksel	bank2		;change from bank3 to bank2
		movf	eedata,f	;set z flag if data is zero
		banksel	bank0		;change from bank2 to bank0
		btfss	status,z	;test z flag
traperror2:	goto	traperror2	;if not zero then is no valid code
		goto	startusercode	;if zero then run user code

;-----------------------------------------------------------------------------
;start of routine to load and program new code

loader:		clrf	testbyte	;indicate no reset vector code yet

		call	loadstatusaddr	;load address of codestatus word
		movlw	0x3f		;load data to indicate no program
		movwf	eedath
		movlw	0xff		;load data to indicate no program
		movwf	eedata
		call	flashwrite	;write new codestatus word

		call	setup		;set up serial port


;-----------------------------------------------------------------------------
;get new line of hex file starting with ':'
;get first 8 bytes after ':' and extract address and number of bytes

getnewline:	call	serialreceive	;get new byte from serial port
		xorlw	':'		;check if ':' received
		btfss	status,z
		goto	getnewline	;if not then wait for next byte

		clrf	checksum	;start with checksum zero

		call	gethexbyte	;get number of program data bytes in line
		andlw	0x1f		;limit number in case of error in file
		movwf	numwords
		bcf	status,c
		rrf	numwords,f	;divide by 2 to get number of words

		call	gethexbyte	;get upper half of program start address
		movwf	addressh

		call	gethexbyte	;get lower half of program start address
		movwf	addressl

		bcf	status,c
		rrf	addressh,f	;divide address by 2 to get word address
		rrf	addressl,f

		call	gethexbyte	;get record type
		xorlw	0x01
		btfsc	status,z	;check if end of file record (0x01)
		goto	filedone	;if end of file then all done

		movf	hexbyte,w
		xorlw	0x00
		btfss	status,z	;check if regular line record (0x00)
		goto	linedone	;if not then ignore line and send '.'

		movlw	0xe0
		addwf	addressh,w	;check if address < 0x2000
		btfsc	status,c	;which is id locations and config bits
		goto	linedone	;if so then ignore line and send '.'

;-----------------------------------------------------------------------------
;get data bytes and checksum from line of hex file

		movlw	dataarray
		movwf	fsr		;set pointer to start of array
		movf	numwords,w
		movwf	counter		;set counter to number of words

getdata:	call	gethexbyte	;get low data byte
		movwf	indf		;save in array
		incf	fsr,f		;point to high byte

		call	gethexbyte	;get high data byte
		movwf	indf		;save in array
		incf	fsr,f		;point to next low byte

		decfsz	counter,f
		goto	getdata

		call	gethexbyte	;get checksum
		movf	checksum,w	;check if checksum correct
		btfss	status,z
		goto	errormessage

		bsf	rts_output 	;set rts off to stop data being received

;-----------------------------------------------------------------------------
;get saved data one word at a time to program into flash 

		movlw	dataarray
		movwf	fsr		;point to start of array
		movf	numwords,w
		movwf	counter		;set counter to half number of bytes

;-----------------------------------------------------------------------------
;check if address is in reset code area

checkaddress:	movf	addressh,w	;checking for boot location code
		btfss	status,z	;test if addressh is zero 
		goto	checkaddress1	;if not go check if reset code received

		movlw	0xfc	
		addwf	addressl,w	;add 0xfc (-4) to address
		btfsc	status,c	;no carry means address < 4
		goto	checkaddress1	;if not go check if reset code received

		bsf	testbyte,0	;show that reset vector code received
		movf	addressl,w	;relocate addresses 0-3 to new location
		addlw	low (startusercode + 1) ;add low address to new location
		banksel	bank2			;change from bank0 to bank2
		movwf	eeadr		;load new low address
		movlw	high (startusercode + 1) ;get new location high address
		movwf	eeadrh		;load high address
		goto	loaddata	;go get data byte and program into flash

;-----------------------------------------------------------------------------
;check if reset code has been received
;check if address is too high and conflicts with boot loader

checkaddress1:	btfss	testbyte,0	;check if reset vector code received first
		goto	errormessage	;if not then error

		movlw	high startofboot ;get high byte of address
		subwf	addressh,w
		btfss	status,c	;test if less than boot code address 
		goto	loadaddress	;yes so continue with write
		btfss	status,z	;test if equal to boot code address 
		goto	errormessage	;no so error in high byte of address

		movlw	low startofboot	;get low byte of address
		subwf	addressl,w
		btfsc	status,c	;test if less than boot code address 
		goto	errormessage	;no so error in address

;-----------------------------------------------------------------------------
;load address and data and write data into flash

loadaddress:	movf	addressh,w	;get high address
		banksel	bank2			;change from bank0 to bank2
		movwf	eeadrh		;load high address
		banksel	bank0			;change from bank2 to bank0
		movf	addressl,w	;get low address
		banksel	bank2			;change from bank0 to bank2
		movwf	eeadr		;load low address

loaddata:	movf	indf,w		;get low byte from array
		movwf	eedata		;load low byte
		incf	fsr,f		;point to high data byte
		movf	indf,w		;get high byte from array
		movwf	eedath		;load high byte
		incf	fsr,f		;point to next low data byte

		call	flashwrite	;write data to program memory

		banksel	bank0			;change from bank3 to bank0
		incfsz	addressl,f	;increment low address byte
		goto	checklinedone	;check for rollover
		incf	addressh,f	;if so then increment high address byte

checklinedone:	decfsz	counter,f	;check if all words have been programmed
		goto	checkaddress	;if not then go program next word

;-----------------------------------------------------------------------------
;done programming line of file

linedone:	movlw	'.'		;line has been programmed so
		call	serialtransmit	;transmit progress indicator back
		goto	getnewline	;go get next line hex file

;-----------------------------------------------------------------------------
;done programming file so send success indicator and trap execution until reset

filedone:	movlw	'O'		;programming complete so
		call	serialtransmit	;transmit success indicator back
		movlw	'K'		;programming complete so
		call	serialtransmit	;transmit success indicator back

		call	loadstatusaddr	;load address of codestatus word
		clrf	eedath		;load data to indicate program exists
		clrf	eedata		;load data to indicate program exists
		call	flashwrite
trapfiledone:	goto	trapfiledone	;all done so wait for reset

;-----------------------------------------------------------------------------
;error in hex file so send failure indicator and trap error

errormessage:	movlw	'E'		;error occurred so
		call	serialtransmit	;transmit failure indicator back
		movlw	'R'		;error occurred so
		call	serialtransmit	;transmit failure indicator back
		movlw	'R'		;error occurred so
		call	serialtransmit	;transmit failure indicator back
		movlw	'O'		;error occurred so
		call	serialtransmit	;transmit failure indicator back
		movlw	'R'		;error occurred so
		call	serialtransmit	;transmit failure indicator back
traperror3:	goto	traperror3	;trap error and wait for reset

;-----------------------------------------------------------------------------
;load address of codestatus word into flash memory address registers
;this routine returns in bank2

loadstatusaddr:
		banksel	bank2			;change from bank0 to bank2
		movlw	high codestatus	;load high addr of codestatus location
		movwf	eeadrh
		movlw	low codestatus	;load low addr of codestatus location
		movwf	eeadr
		return

;-----------------------------------------------------------------------------
;receive two ascii digits and convert into one hex byte
;this routine returns in bank0

gethexbyte:	call	serialreceive	;get new byte from serial port
		addlw	0xbf		;add -'a' to ascii high byte
		btfss	status,c	;check if positive
		addlw	0x07		;if not, add 17 ('0' to '9')
		addlw	0x0a		;else add 10 ('a' to 'f') 
		movwf	hexbyte		;save nibble
		swapf	hexbyte,f	;move nibble to high position

		call	serialreceive	;get new byte from serial port
		addlw	0xbf		;add -'a' to ascii low byte
		btfss	status,c	;check if positive
		addlw	0x07		;if not, add 17 ('0' to '9')
		addlw	0x0a		;else add 10 ('a' to 'f') 
		iorwf	hexbyte,f	;add low nibble to high nibble
		movf	hexbyte,w	;put result in w reg
		addwf	checksum,f	;add to cumulative checksum
		return

;=========================================================================
;Initialisieren der UART Port RC4 bis RC7 
;=========================================================================

setup:
	banksel	bank0		;
	bsf	a_rts           ;RTS aus
	banksel	bank1		;
	bcf	trisc,4         ;RTS Pin wird Ausgang

	btfss	e_pictakt	;mit welcher Taktfrequenz arbeitet der PIC?
	goto	s4mhz

s10mhz:
				;10 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_38400	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_57600	;Setze Baudrate
	goto	setserialrate

s4mhz:
				;4 MHz Takt
	btfss	e_baud		;kleine Baudrate?
	movlw	baud_9600	;Setze Baudrate
	btfsc	e_baud		;grosse Baudrate?
	movlw	baud_19200	;Setze Baudrate

setserialrate:
	movwf	spbrg
	bsf	txsta,brgh	;baud rate high speed option
	bsf	txsta,txen	;enable transmission
	banksel	bank0		;
	bsf	rcsta,cren	;enable reception
	bsf	rcsta,spen	;enable serial port

	movlw	'N'		;
	call	serialtransmit	;transmit char
	movlw	'W'		;
	call	serialtransmit	;transmit char
	movlw	'T'		;
	call	serialtransmit	;transmit char
	movlw	'7'		;
	call	serialtransmit	;transmit char

	return

;-----------------------------------------------------------------------------
;wait for byte to be received in usart and return with byte in w
;this routine returns in bank0

serialreceive:
		banksel	bank0		;change from unknown bank to bank0
		bcf	rts_output   	;set rts on for data to be received
		btfss	pir1,rcif	;check if data received
		goto	$-1		;wait until new data
		movf	rcreg,w		;get received data into w
		return

;-----------------------------------------------------------------------------
;transmit byte in w register from usart
;this routine returns in bank0

serialtransmit:
		banksel	bank0		;change from unknown bank to bank0
		btfsc	cts_input	;check cts to see if data can be sent
		goto	$-1
		btfss	pir1,txif	;check that buffer is empty
		goto	$-1
		movwf	txreg		;transmit byte
		return

;-----------------------------------------------------------------------------
;write to a location in the flash program memory
;address in eeadrh and eeadr, data in eedath and eedata
;this routine returns in bank3

flashwrite:
		banksel	bank3		;change from bank2 to bank3
		movlw	0x84		;enable writes to program flash
		movwf	eecon1

		movlw	0x55		;do timed access writes
		movwf	eecon2
		movlw	0xaa
		movwf	eecon2
		bsf	eecon1,wr	;begin writing to flash

		nop			;processor halts here while writing
		nop
		return

;-----------------------------------------------------------------------------
;read from a location in the flash program memory
;address is in eeadrh and eeadr, data returned in eedath and eedata
;routine is only called once and can be placed in-line saving a call and return
;this routine returns in bank3 and is called when in bank2

flashread:	movlw	0x1f		;keep address within range
		andwf	eeadrh,f

		banksel	bank3			;change from bank2 to bank3
		movlw	0x80		;enable reads from program flash
		movwf	eecon1

		bsf	eecon1,rd	;read from flash

		nop			;processor waits while reading
		nop
		return

;-----------------------------------------------------------------------------

		end

